import { jsonAgents } from './agents.js';
import { jsonButtons } from './buttons.js';

const BASE_URL = 'https://api.siliconflow.cn/v1';

const API_KEY = 'sk-xx';

const CHAT_COMPLETIONS_ENDPOINT = '/chat/completions';
const MODELS_ENDPOINT = '/models';

const panelWidth = 430;
const panelHeight = 580;
const sidePanelWidth = 350;

const htmlElement = document.documentElement;
const wordParam = document.getElementById('word-param');
const gdBlockTranslation = document.getElementById('gd-block-translation');
const primaryButton = document.getElementById('primary-button');

const popupContainer = document.getElementById('popup-container');
const popupDropdown = document.getElementById('popup-dropdown');
const copyButton = document.getElementById('copy-button');
const aiSearchButton = document.getElementById('ai-search-button');
const singleTranslateButton = document.getElementById('single-translate-button');
const batchTranslateButton = document.getElementById('batch-translate-button');
const bookmarkButton = document.getElementById('bookmark-button');
const moreButton = document.getElementById('more-button');

const aiPanel = document.getElementById('ai-panel');
const chatroomMainPanel = document.getElementById('chatroom-main-panel');
const historyMainPanel = document.getElementById('history-main-panel');
const agentMainPanel = document.getElementById('agent-main-panel');

const mainColumn = document.getElementById('main-column');
const panelContainers = document.getElementsByClassName('panel-container');
const mainPanels = document.getElementsByClassName('main-panel');
const sidePanels = document.getElementsByClassName('side-panel');
const agentSidePanel = document.getElementById('agent-side-panel');

const chatroomMainPanelBody = document.getElementById('chatroom-main-panel-body');
const historyMainPanelBody = document.getElementById('history-main-panel-body');
const translationMainPanelBody = document.getElementById('translation-main-panel-body');
const agentMainPanelBody = document.getElementById('agent-main-panel-body');
const settingMainPanelBody = document.getElementById('setting-main-panel-body');

const chatroomContent = document.getElementById('chatroom-content');
const historyContent = document.getElementById('history-content');
const translationContent = document.getElementById('translation-content');
const agentContent = document.getElementById('agent-content');
const settingContent = document.getElementById('setting-content');

const aiPanelHeader = document.getElementById('ai-panel-header');
const dockButton = document.getElementById('dock-button');
const maximizeButton = document.getElementById('maximize-button');
const dragButton = document.getElementById('drag-button');
const closeButton = document.getElementById('close-button');

const sidebar = document.getElementById('sidebar');
const tabButtons = document.getElementsByClassName('tab-button');
const chatroomTabButton = document.getElementById('chatroom-tab-button');
const historyTabButton = document.getElementById('history-tab-button');
const translationTabButton = document.getElementById('translation-tab-button');
const agentTabButton = document.getElementById('agent-tab-button');
const settingTabButton = document.getElementById('setting-tab-button');

const inputArea = document.getElementById('input-area');
const sendButton = document.getElementById('send-button');
const stopButton = document.getElementById('stop-button');
const newButton = document.getElementById('new-button');

const agentOverview = document.getElementById('agent-overview');
const agentOverviewAvatar = document.getElementById('agent-overview-avatar');
const agentOverviewTitle = document.getElementById('agent-overview-title');
const agentOverviewTags = document.getElementById('agent-overview-tags');
const agentOverviewDescription = document.getElementById('agent-overview-description');
const activateAgentButton = document.getElementById('activate-agent-button');
const agentOverviewAuthor = document.getElementById('agent-overview-author');
const agentOverviewDate = document.getElementById('agent-overview-date');
const agentOverviewPrompt = document.getElementById('agent-overview-prompt');

const modelDropdown = document.getElementById('model-dropdown');
const modelDropdownButton = document.getElementById('model-dropdown-button');
const modelDropdownMenu = document.getElementById('model-dropdown-menu');

const modelDropdownItems = [];
const currentMessages = [];
const historyMessages = [];

let modelInUse;
let aiPanelState;
let aiPanelTopPrev;
let aiPanelRightPrev;
let dockState;
let maximizeState;
let offsetX, offsetY;

function setModelInUse(model) {
    modelInUse = model;
    modelDropdownButton.textContent = modelInUse;
    modelDropdownItems.forEach(modelDropdownItem => {
        modelDropdownItem.classList.remove('active');
        if (modelDropdownItem.getAttribute('data-model') === model) {
            modelDropdownItem.classList.add('active');
            localStorage.setItem('modelInUse', modelInUse);
        }
    });
}

function setModels(modelData) {
    modelDropdownItems.forEach(modelDropdownItem => {
        modelDropdownItem.remove();
    });
    modelDropdownItems.splice(0, modelDropdownItems.length);

    localStorage.setItem('modelData', JSON.stringify(modelData));

    const models = [];
    modelData.data.forEach(item => {
        models.push(item.id);
    });
    models.sort();
    models.forEach(model => {
        const modelDropdownItem = document.createElement('button');
        modelDropdownItem.className = 'model-dropdown-item';
        modelDropdownItem.setAttribute('data-model', model);
        modelDropdownItem.textContent = model;
        modelDropdownMenu.appendChild(modelDropdownItem);
        modelDropdownItems.push(modelDropdownItem);
    });

    setModelInUse(localStorage.getItem('modelInUse') || models[0]);

    modelDropdownItems.forEach(modelDropdownItem => {
        modelDropdownItem.addEventListener('click', function (event) {
            setModelInUse(this.getAttribute('data-model'));
            modelDropdownMenu.classList.remove('active');
        });
    });
}

function saveHistoryMessages(historyMessages) {
    localStorage.setItem('historyMessages', JSON.stringify(historyMessages));
}

function getSelectedText() {
    const selection = window.getSelection();
    const selectedText = selection.toString().trim();
    return selectedText;
}

function getContextElement() {
    const selection = window.getSelection();
    if (selection.rangeCount === 0) {
        return null;
    }
    const range = selection.getRangeAt(0);
    let contextElement = range.commonAncestorContainer;
    if (contextElement.nodeType === Node.TEXT_NODE) {
        contextElement = contextElement.parentNode;
    }
    return contextElement;
}

function getContextText() {
    return getContextElement().textContent;
}

function mouseMoveHandler(event) {
    if (event.clientX < 0 || event.clientX > document.documentElement.clientWidth || event.clientY < 0 || event.clientY > document.documentElement.clientHeight) {
        return;
    }

    const right = document.documentElement.clientWidth - (event.clientX - offsetX);
    const top = event.clientY - offsetY;
    aiPanel.style.top = `${top}px`;
    aiPanel.style.right = `${right}px`;

    aiPanelTopPrev = aiPanel.style.top;
    aiPanelRightPrev = aiPanel.style.right;
    localStorage.setItem('aiPanelTopPrev', aiPanelTopPrev);
    localStorage.setItem('aiPanelRightPrev', aiPanelRightPrev);
}

function mouseUpHandler() {
    aiPanel.style.transition = 'all 0.2s';
    document.removeEventListener('mousemove', mouseMoveHandler);
    document.removeEventListener('mouseup', mouseUpHandler);
}

function showPopupContainer() {
    popupContainer.classList.add('active');
    //popupDropdown.classList.remove('active');
}

function hidePopupContainer() {
    popupContainer.classList.remove('active');
    popupDropdown.classList.remove('active');
}

function setDockState() {
    dockState = true;
    dockButton.classList.add('active');
    localStorage.setItem('dockState', dockState.toString());
}

function clearDockState() {
    dockState = false;
    dockButton.classList.remove('active');
    localStorage.setItem('dockState', dockState.toString());
}

function setMaximizeState() {
    maximizeState = true;
    maximizeButton.classList.add('active');
    localStorage.setItem('maximizeState', maximizeState.toString());
}

function clearMaximizeState() {
    maximizeState = false;
    maximizeButton.classList.remove('active');
    localStorage.setItem('maximizeState', maximizeState.toString());
}

function getPanelState() {
    let panelState;
    if (window.getComputedStyle(aiPanel).visibility === 'hidden') {
        panelState = 'minimized';
    } else {
        if (aiPanel.style.width === '100%') {
            panelState = 'maximized';
        } else {
            if (aiPanel.style.height === '100%') {
                panelState = 'docked';
            } else {
                panelState = 'floating';
            }
        }
    }
    return panelState;
}

function maximizePanel() {
    const panelState = getPanelState();
    if (panelState !== 'minimized' && panelState !== 'docked') {
        aiPanelTopPrev = aiPanel.style.top;
        aiPanelRightPrev = aiPanel.style.right;
        localStorage.setItem('aiPanelTopPrev', aiPanelTopPrev);
        localStorage.setItem('aiPanelRightPrev', aiPanelRightPrev);
    }
    aiPanel.classList.add('active');
    aiPanel.style.top = '0';
    aiPanel.style.right = '0';
    aiPanel.style.width = '100%';
    aiPanel.style.height = '100%';
    htmlElement.style.marginRight = '0px';
    htmlElement.style.width = '100%';
    setMaximizeState();
    clearDockState();
    dragButton.disabled = true;
}

function minimizePanel() {
    aiPanel.classList.remove('active');
    htmlElement.style.marginRight = '0px';
    htmlElement.style.width = '100%';
}

function floatPanel() {
    aiPanel.classList.add('active');
    aiPanel.style.top = `${aiPanelTopPrev}`;
    aiPanel.style.right = `${aiPanelRightPrev}`;
    aiPanel.style.width = `${panelWidth}px`;
    aiPanel.style.height = `${panelHeight}px`;
    htmlElement.style.marginRight = '0px';
    htmlElement.style.width = '100%';
    clearMaximizeState();
    clearDockState();
    dragButton.disabled = false;
}

function dockPanel() {
    aiPanel.classList.add('active');
    aiPanel.style.top = '0';
    aiPanel.style.right = '0';
    aiPanel.style.width = `${panelWidth}px`;
    aiPanel.style.height = '100%';
    htmlElement.style.marginRight = `${panelWidth}px`;
    htmlElement.style.width = `calc(100% - ${panelWidth}px)`;
    clearMaximizeState();
    setDockState();
    dragButton.disabled = true;
}

function showPanel() {
    const panelState = getPanelState();
    if (panelState !== 'minimized') {
        return;
    }
    if (maximizeState) {
        maximizePanel();
    } else {
        if (dockState) {
            dockPanel();
        } else {
            floatPanel();
        }
    }
}

function switchToTab(dataTab) {
    for (const tabButton of tabButtons) {
        if (tabButton.getAttribute('data-tab') === dataTab) {
            tabButton.classList.add('active');
        } else {
            tabButton.classList.remove('active');
        }
    }
    for (const panelContainer of panelContainers) {
        if (panelContainer.id === dataTab) {
            panelContainer.classList.add('active');
        } else {
            panelContainer.classList.remove('active');
        }
    }
}

function handleSidePanelPosition(entries) {
    for (const entry of entries) {
        if (entry.contentRect.width > (sidePanelWidth + panelWidth)) {
            for (const sidePanel of sidePanels) {
                if (sidePanel.classList.contains('active')) {
                    sidePanel.style.position = 'static';
                } else {
                    sidePanel.style.position = 'absolute';
                }
            }
        } else {
            for (const sidePanel of sidePanels) {
                sidePanel.style.position = 'absolute';
            }
        }
    }
}

function triggerResize() {
    handleSidePanelPosition([{ contentRect: mainColumn.getBoundingClientRect() }]);
}

function toggleSidePanel(dataTab) {
    const sidePanel = document.querySelector(`#${dataTab} .side-panel`);
    sidePanel.classList.toggle('active');
    triggerResize();
}

function startNewChat() {
    dragButton.textContent = '';
    const messageElements = document.querySelectorAll('#chatroom-content .message');
    messageElements.forEach(element => {
        element.parentNode.removeChild(element);
    });
    currentMessages.splice(0, currentMessages.length);
}

function addMessageText(messageText, role, container = undefined) {
    const messageElement = document.createElement('div');
    messageElement.classList.add('message', role);
    const messageContent = document.createElement('div');
    messageContent.classList.add('message-content');
    messageContent.classList.add('markdown-body');
    switch (role) {
        case 'system':
            messageContent.innerHTML = messageText;
            break;
        case 'assistant':
            messageContent.innerHTML = messageText;
            break;
        case 'user':
            messageContent.innerText = messageText;
            break;
        default:
    }

    const hoverButton = document.createElement('button');
    hoverButton.classList.add('hover-button');
    hoverButton.innerHTML = '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="20" height="20" color="#000000" fill="none"><path d="M9 15C9 12.1716 9 10.7574 9.87868 9.87868C10.7574 9 12.1716 9 15 9L16 9C18.8284 9 20.2426 9 21.1213 9.87868C22 10.7574 22 12.1716 22 15V16C22 18.8284 22 20.2426 21.1213 21.1213C20.2426 22 18.8284 22 16 22H15C12.1716 22 10.7574 22 9.87868 21.1213C9 20.2426 9 18.8284 9 16L9 15Z" stroke="currentColor" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round" /><path d="M16.9999 9C16.9975 6.04291 16.9528 4.51121 16.092 3.46243C15.9258 3.25989 15.7401 3.07418 15.5376 2.90796C14.4312 2 12.7875 2 9.5 2C6.21252 2 4.56878 2 3.46243 2.90796C3.25989 3.07417 3.07418 3.25989 2.90796 3.46243C2 4.56878 2 6.21252 2 9.5C2 12.7875 2 14.4312 2.90796 15.5376C3.07417 15.7401 3.25989 15.9258 3.46243 16.092C4.51121 16.9528 6.04291 16.9975 9 16.9999" stroke="currentColor" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round" /></svg>';
    messageElement.appendChild(hoverButton);
    hoverButton.addEventListener('click', function (event) {
        navigator.clipboard.writeText(this.parentElement.textContent);
    });

    messageElement.appendChild(messageContent);
    if (container !== undefined) {
        container.appendChild(messageElement);
        container.parentElement.scrollTop = container.parentElement.scrollHeight;
    }
}

async function addPresetMessages(messages) {
    sendButton.disabled = true;
    newButton.disabled = true;
    sendButton.classList.remove('active');
    stopButton.classList.add('active');

    for (const message of messages) {
        currentMessages.push(message);
        historyMessages.push(message);
        const messageText = message.content;
        switch (message.role) {
            case 'system':
                addMessageText(marked.parse(messageText), 'system', chatroomContent);
                addMessageText(marked.parse(messageText), 'system', historyContent);
                hljs.highlightAll();
                break;
            case 'assistant':
                addMessageText(marked.parse(messageText), 'assistant', chatroomContent);
                addMessageText(marked.parse(messageText), 'assistant', historyContent);
                hljs.highlightAll();
                break;
            case 'user':
                addMessageText(messageText, 'user', chatroomContent);
                addMessageText(messageText, 'user', historyContent);
                break;
            default:
        }
    }
    addMessageText('思考中...', 'assistant', chatroomContent);
    addMessageText('思考中...', 'assistant', historyContent);

    const response = await fetchResponse(currentMessages);
    let lastElement = document.querySelector('#chatroom-content .message.assistant:last-child .message-content');
    const assistantMessageText = await displayStream(response, lastElement, chatroomMainPanelBody, true);
    lastElement = document.querySelector('#history-content .message.assistant:last-child .message-content');
    displayStream(assistantMessageText, lastElement, historyMainPanelBody, true);
    currentMessages.push({ 'role': 'assistant', 'content': assistantMessageText });
    historyMessages.push({ 'role': 'assistant', 'content': assistantMessageText });
    saveHistoryMessages(historyMessages);

    sendButton.disabled = false;
    newButton.disabled = false;
    sendButton.classList.add('active');
    stopButton.classList.remove('active');
}

async function fetchResponse(messages) {
    const url = BASE_URL + CHAT_COMPLETIONS_ENDPOINT;
    const body = JSON.stringify({
        model: modelInUse,
        messages: messages,
        stream: true,
        /*
        max_tokens: 4096,
        temperature: 0.7,
        top_p: 0.7,
        //top_k: 50,
        frequency_penalty: 0.5,
        n: 1
        */
    });
    const options = {
        method: 'POST',
        headers: {
            'accept': 'application/json',
            'content-type': 'application/json',
            'authorization': `Bearer ${API_KEY}`
        },
        body: body
    };
    const response = await fetch(url, options);

    return response
}

async function displayStream(response, element = undefined, scrollContainer = undefined, convert) {
    let accumulatedText = '';
    let textCopy = '';
    if (typeof (response) === 'string') {
        textCopy = response;
        if (convert) {
            textCopy = marked.parse(textCopy);
        }
        if (element !== undefined) {
            element.innerHTML = textCopy;
        }
        if (scrollContainer !== undefined) {
            scrollContainer.scrollTop = scrollContainer.scrollHeight;
        }
        hljs.highlightAll();
        return response;
    }

    const reader = response.body.getReader();
    const decoder = new TextDecoder('utf-8');
    while (true) {
        const { done, value } = await reader.read();
        if (done) {
            console.log('Stream complete');
            hljs.highlightAll();
            break;
        }
        const chunk = decoder.decode(value, { stream: true });
        const lines = chunk.split('\n');
        if (!response.ok) {
            lines.forEach(line => {
                accumulatedText += line;
                textCopy = accumulatedText;
                if (convert) {
                    textCopy = marked.parse(textCopy);
                }
                if (element !== undefined) {
                    element.innerHTML = textCopy;
                }
                if (scrollContainer !== undefined) {
                    scrollContainer.scrollTop = scrollContainer.scrollHeight;
                }
            });
        } else {
            lines.forEach(line => {
                if (line.startsWith('data: ')) {
                    const jsonString = line.slice(6);
                    try {
                        const data = JSON.parse(jsonString);
                        const content = data.choices[0].delta.content;
                        if (content) {
                            accumulatedText += content;
                            textCopy = accumulatedText;
                            if (convert) {
                                textCopy = marked.parse(textCopy);
                            }
                            if (element !== undefined) {
                                element.innerHTML = textCopy;
                            }
                            if (scrollContainer !== undefined) {
                                //scrollContainer.scrollTop = scrollContainer.scrollHeight;
                                scrollContainer.scrollTo({
                                    top: scrollContainer.scrollHeight,
                                    behavior: 'smooth'
                                });
                            }
                            //hljs.highlightAll();
                        }
                    } catch (e) {
                        //console.error('Error:', e);
                    }
                }
            });
        }
    }
    return accumulatedText;
}

function gdWordTranslation() {
    const queryString = window.location.search;
    const urlParams = new URLSearchParams(queryString);
    const word = urlParams.get('word');
    if (!word) {
        return;
    }
    wordParam.textContent = word;

    const toLang = 'Simplified Chinese'
    const srcText = word;
    const messages = [
        {
            'role': 'system',
            'content': 'You are a professional, authentic machine translation engine.'
        },
        {
            'role': 'user',
            'content': `Translate the following source text in tripple quotes to ${toLang}, output translation directly without any additional text.\nSource Text:\n"""${srcText}"""`
        }
    ];
    (async () => {
        const response = await fetchResponse(messages);
        displayStream(response, gdBlockTranslation, undefined, false);
    })();
}



aiPanelTopPrev = localStorage.getItem('aiPanelTopPrev') || '0px';
aiPanelRightPrev = localStorage.getItem('aiPanelRightPrev') || '0px';
aiPanel.style.top = aiPanelTopPrev;
aiPanel.style.right = aiPanelRightPrev;

dockState = JSON.parse(localStorage.getItem('dockState') || 'false');
maximizeState = JSON.parse(localStorage.getItem('maximizeState') || 'false');

try {
    historyMessages.push(...JSON.parse(localStorage.getItem('historyMessages')));
} catch (e) { }

historyMessages.forEach(message => {
    switch (message.role) {
        case 'system':
            addMessageText(marked.parse(message.content), 'system', historyContent);
            hljs.highlightAll();
            break;
        case 'assistant':
            addMessageText(marked.parse(message.content), 'assistant', historyContent);
            hljs.highlightAll();
            break;
        case 'user':
            addMessageText(message.content, 'user', historyContent);
            break;
        default:

    }
});

(function initAi() {
    primaryButton.disabled = true;
    const url = BASE_URL + MODELS_ENDPOINT;
    fetch(url, {
        method: 'GET',
        headers: {
            'accept': 'application/json',
            'authorization': `Bearer ${API_KEY}`
        }
    }).then(response => response.json()).then(data => {
        setModels(data);
        primaryButton.disabled = false;
        gdWordTranslation();
    }).catch(err => console.error(err));
})();

for (const jsonButton of jsonButtons) {
    const button = document.createElement('button');
    button.classList.add('popup-dropdown-item');
    button.classList.add(jsonButton.id);
    const parserDiv = document.createElement('div');
    parserDiv.innerHTML = jsonButton.icon;
    const svg = parserDiv.querySelector('svg');
    button.appendChild(svg);
    const name = document.createElement('div');
    name.textContent = jsonButton.name;
    button.appendChild(name)
    popupDropdown.appendChild(button);

    button.addEventListener('click', function () {
        setTimeout(() => {
            hidePopupContainer();
        }, 0);
        showPanel();
        switchToTab('chatroom-tab');
        startNewChat();

        const messages = [];
        for (const message of jsonButton.messages) {
            const messageCopy = JSON.parse(JSON.stringify(message));
            let messageText = messageCopy.content;
            messageText = messageText.replace('{selectedText}', getSelectedText());
            messageText = messageText.replace('{contextText}', getContextText());
            messageCopy.content = messageText;
            messages.push(messageCopy);
        }
        addPresetMessages(messages);
    });
}

for (const jsonAgent of jsonAgents) {
    const agentCard = document.createElement('div');
    const agentAvatar = document.createElement('div');
    const agentTitle = document.createElement('div');
    const agentCardHeader = document.createElement('div');
    const agentCardBody = document.createElement('div');
    const agentCardFooter = document.createElement('div');
    agentCard.className = 'agent-card';
    agentCard.setAttribute('data-title', jsonAgent.meta.title);
    agentAvatar.className = 'agent-avatar';
    agentAvatar.textContent = jsonAgent.meta.avatar;
    agentTitle.className = 'agent-title';
    agentTitle.textContent = jsonAgent.meta.title;

    agentCardHeader.className = 'agent-card-header';
    agentCardBody.className = 'agent-card-body';
    agentCardBody.textContent = jsonAgent.meta.description;
    agentCardFooter.className = 'agent-card-footer';

    for (const tag of jsonAgent.meta.tags) {
        const agentTag = document.createElement('div');
        agentTag.className = 'agent-tag';
        agentTag.textContent = tag;
        agentCardFooter.appendChild(agentTag);
    }

    agentCardHeader.appendChild(agentTitle);
    agentCardHeader.appendChild(agentAvatar);
    agentCard.appendChild(agentCardHeader);
    agentCard.appendChild(agentCardBody);
    agentCard.appendChild(agentCardFooter);
    agentContent.appendChild(agentCard);

    agentCard.addEventListener('click', function () {
        agentOverview.setAttribute('data-identifier', jsonAgent.identifier);
        agentOverviewAvatar.textContent = jsonAgent.meta.avatar;
        agentOverviewTitle.textContent = jsonAgent.meta.title;

        agentOverviewTags.innerHTML = '';
        for (const tag of jsonAgent.meta.tags) {
            const agentOverviewTag = document.createElement('div');
            agentOverviewTag.className = 'agent-tag';
            agentOverviewTag.textContent = tag;
            agentOverviewTags.appendChild(agentOverviewTag);
        }

        agentOverviewDescription.textContent = jsonAgent.meta.description;
        agentOverviewAuthor.textContent = `@${jsonAgent.author}`;
        agentOverviewAuthor.setAttribute('href', jsonAgent.homepage);
        agentOverviewDate.textContent = jsonAgent.createAt;
        agentOverviewPrompt.innerHTML = marked.parse(jsonAgent.config.systemRole);
        hljs.highlightAll();

        agentSidePanel.classList.add('active');
        triggerResize();
    });
}

primaryButton.addEventListener('click', function () {
    const panelState = getPanelState();
    if (panelState === 'minimized') {
        showPanel();
    } else {
        minimizePanel();
    }
});

dragButton.addEventListener('mousedown', (event) => {
    aiPanel.style.transition = 'none';
    const rect = aiPanel.getBoundingClientRect();
    offsetX = event.clientX - rect.right;
    offsetY = event.clientY - rect.top;
    document.addEventListener('mousemove', mouseMoveHandler);
    document.addEventListener('mouseup', mouseUpHandler);
});

copyButton.addEventListener('click', function () {
    const selection = window.getSelection();
    const range = selection.getRangeAt(0);
    const rect = range.getBoundingClientRect();
    const selectedText = selection.toString();
    navigator.clipboard.writeText(selectedText);
    setTimeout(() => {
        hidePopupContainer();
    }, 0);
    return;
});

singleTranslateButton.addEventListener('click', async function () {
    setTimeout(() => {
        hidePopupContainer();
    }, 0);

    const contextElement = getContextElement();
    if (!contextElement || contextElement.classList.contains('gd-inline-translation') || contextElement.classList.contains('gd-block-translation')) {
        return;
    }

    const inlineTranslationElementList = contextElement.querySelectorAll('.gd-inline-translation');
    for (let i = 0; i < inlineTranslationElementList.length; i++) {
        inlineTranslationElementList[i].remove();
    }
    const srcText = contextElement.innerText;

    const textNode = document.createTextNode('Loading...');
    const newElement = document.createElement('span');
    newElement.classList.add('gd-inline-translation');
    newElement.appendChild(textNode);
    contextElement.appendChild(newElement);

    const toLang = 'Simplified Chinese'
    const messages = [
        {
            'role': 'system',
            'content': 'You are a professional translation engine, please translate the text into a colloquial, professional, elegant and fluent content, without the style of machine translation. You must only translate the text content, never interpret it.'
        },
        {
            'role': 'user',
            'content': `Translate into ${toLang}:\n"""${srcText}"""`
        }
    ];
    const response = await fetchResponse(messages);
    displayStream(response, newElement, undefined, false);

});

batchTranslateButton.addEventListener('click', async function () {
    setTimeout(() => {
        hidePopupContainer();
    }, 0);

    const contextElement = getContextElement();
    if (!contextElement) {
        return;
    }
    if (contextElement.classList.contains('gd-inline-translation')) {
        return;
    }

    let selectorStrAll = '';
    selectorStrAll = contextElement.tagName;
    for (let i = 0; i < contextElement.classList.length; i++) {
        selectorStrAll = selectorStrAll + '.' + contextElement.classList[i];
    }
    let selectorStrArray = [];
    selectorStrArray.push(contextElement.tagName);
    for (let i = 0; i < contextElement.classList.length; i++) {
        selectorStrArray.push('.' + contextElement.classList[i]);
    }

    let identicalElementList = document.querySelectorAll(selectorStrAll);
    identicalElementList = Array.from(identicalElementList).filter(function (e) {
        return !selectorStrArray.some(function (selectorStr) {
            return Array.from(e.querySelectorAll(selectorStr)).length > 0;
        });
    });

    let requestCount = 0;
    const requestLimit = 10;
    const delay = 700;

    for (let i = 0; i < identicalElementList.length; i++) {
        const identicalElement = identicalElementList[i];
        const srcText = identicalElement.innerText;
        const inlineTranslationElementList = identicalElement.querySelectorAll('.gd-inline-translation');
        for (let j = 0; j < inlineTranslationElementList.length; j++) {
            inlineTranslationElementList[j].remove();
        }

        const textNode = document.createTextNode('Loading...');
        const newElement = document.createElement('span');
        newElement.classList.add('gd-inline-translation');
        newElement.appendChild(textNode);
        identicalElement.appendChild(newElement);

        const toLang = 'Simplified Chinese'
        const messages = [
            {
                'role': 'system',
                'content': 'You are a professional translation engine, please translate the text into a colloquial, professional, elegant and fluent content, without the style of machine translation. You must only translate the text content, never interpret it.'
            },
            {
                'role': 'user',
                'content': `Translate into ${toLang}:\n"""${srcText}"""`
            }
        ];
        const response = await fetchResponse(messages);
        displayStream(response, newElement, undefined, false);

        requestCount++;
        if (requestCount % requestLimit === 0) {
            await new Promise(resolve => setTimeout(resolve, delay));
        }
    }


});

moreButton.addEventListener('click', function () {
    popupDropdown.classList.toggle('active');
});

maximizeButton.addEventListener('click', function () {
    const panelState = getPanelState();
    if (panelState === 'maximized') {
        floatPanel();
    } else {
        maximizePanel();
    }
});

dockButton.addEventListener('click', function () {
    const panelState = getPanelState();
    if (panelState === 'docked') {
        floatPanel();
    } else {
        dockPanel();
    }
});

closeButton.addEventListener('click', function () {
    minimizePanel();
});

for (const tabButton of tabButtons) {
    tabButton.addEventListener('click', function () {
        const dataTab = this.getAttribute('data-tab');
        if (this.classList.contains('active')) {
            toggleSidePanel(dataTab);
        } else {
            switchToTab(dataTab);
        }
    });
}

modelDropdownButton.addEventListener('click', function (event) {
    event.stopPropagation();
    modelDropdownMenu.classList.toggle('active');
});

inputArea.addEventListener('input', function () {
    this.style.height = 'auto';
    this.style.height = this.scrollHeight + 'px';
});

let shiftKeyState = false;
inputArea.addEventListener('keydown', function (event) {
    if (document.activeElement !== inputArea) {
        return;
    }
    if (event.key === 'Shift') {
        shiftKeyState = true;
    }
    if (event.key === 'Enter') {
        if (!shiftKeyState) {
            event.preventDefault();
            sendButton.click();
        }
    }
});

inputArea.addEventListener('keyup', function (event) {
    if (document.activeElement !== inputArea) {
        return;
    }
    if (event.key === 'Shift') {
        shiftKeyState = false;
    }
});

sendButton.addEventListener('click', async function () {
    const messageText = inputArea.value;
    inputArea.value = '';
    const event = new Event('input', { bubbles: true });
    inputArea.dispatchEvent(event);
    if (messageText.trim() === '') {
        return;
    }
    addPresetMessages([{ 'role': 'user', 'content': messageText }]);
});

newButton.addEventListener('click', function () {
    inputArea.focus();
    dragButton.textContent = '';
    startNewChat();
});

activateAgentButton.addEventListener('click', async function () {
    switchToTab('chatroom-tab');
    startNewChat();
    dragButton.textContent = agentOverviewTitle.textContent;

    const jsonAgent = jsonAgents.find(item => item.identifier === agentOverview.getAttribute('data-identifier'));
    const messageText = jsonAgent.config.systemRole;
    const messages = [{ 'role': 'system', 'content': messageText }];
    addPresetMessages(messages);
});

document.addEventListener('mouseup', function (event) {
    setTimeout(function () {
        const element = document.elementFromPoint(event.clientX, event.clientY);
        const selection = window.getSelection();
        if (selection) {
            if (selection.rangeCount > 0) {
                const range = selection.getRangeAt(0);
                const rect = range.getBoundingClientRect();
                const selectedText = selection.toString().trim();
                if (selectedText) {
                    showPopupContainer();
                    let left = (rect.left + rect.right - popupContainer.offsetWidth) / 2;
                    if (left < 0) {
                        left = 10;
                    }
                    if ((left + popupContainer.offsetWidth) > window.innerWidth) {
                        left = window.innerWidth - popupContainer.offsetWidth - 10;
                    }
                    popupContainer.style.top = (rect.bottom + window.scrollY + 10) + 'px';
                    popupContainer.style.left = left + 'px';

                } else {
                    hidePopupContainer();
                }
            } else {
                hidePopupContainer();
            }
        } else {
            hidePopupContainer();
        }
    }, 0);
});

document.addEventListener('click', function (event) {
    if (!event.target.closest('.model-dropdown-menu')) {
        modelDropdownMenu.classList.remove('active');
    }
});

const observer = new ResizeObserver(handleSidePanelPosition);
observer.observe(mainColumn);

